// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_corekit_im/model/team_models.dart';
import 'package:netease_corekit_im/service_locator.dart';
import 'package:netease_corekit_im/services/login/login_service.dart';
import 'package:netease_corekit_im/services/message/message_provider.dart';
import 'package:netease_corekit_im/services/team/team_provider.dart';
import 'package:nim_core/nim_core.dart';

class TeamRepo {
  /// 注册群资料变动监听器
  static Stream<List<NIMTeam>> registerTeamUpdateObserver() {
    return NimCore.instance.teamService.onTeamListUpdate;
  }

  /// 查询群组中用户的资料（当前用户设置）
  static Future<TeamWithMember?> queryTeamWithMember(
      String teamId, String accId) async {
    var team = await NimCore.instance.teamService.queryTeam(teamId);
    if (team.isSuccess) {
      var teamMember =
          await NimCore.instance.teamService.queryTeamMember(teamId, accId);
      return TeamWithMember(team.data!, teamMember.data);
    }
    return null;
  }

  /// 查询群组中用户的资料
  static Future<List<UserInfoWithTeam>?> getMemberList(String teamId) {
    return getIt<TeamProvider>().queryMemberList(teamId);
  }

  /// 设置群消息不提醒
  static Future<bool> updateTeamNotify(String teamId, bool mute) async {
    var result = await NimCore.instance.teamService.muteTeam(
        teamId,
        mute
            ? NIMTeamMessageNotifyTypeEnum.mute
            : NIMTeamMessageNotifyTypeEnum.all);
    return result.isSuccess;
  }

  /// 对整个群禁言、解除禁言，对普通成员生效，只有群组、管理员有权限
  static Future<bool> muteAllMembers(String teamId, bool mute) async {
    var result =
        await NimCore.instance.teamService.muteAllTeamMember(teamId, mute);
    return result.isSuccess;
  }

  static Future<bool> _updateTeam(
      String teamId, NIMTeamUpdateFieldRequest request) async {
    var result =
        await NimCore.instance.teamService.updateTeamFields(teamId, request);
    return result.isSuccess;
  }

  /// 查询群资料
  static Future<NIMTeam?> getTeamInfo(String teamId) {
    return NimCore.instance.teamService.queryTeam(teamId).then((result) {
      if (result.isSuccess) {
        return result.data;
      }
      return null;
    });
  }

  /// 更新群头像
  static Future<bool> updateTeamIcon(String teamId, String avatar) {
    NIMTeamUpdateFieldRequest request = NIMTeamUpdateFieldRequest();
    request.setIcon(avatar);
    return _updateTeam(teamId, request);
  }

  /// 更新群名
  static Future<bool> updateTeamName(String teamId, String name) {
    NIMTeamUpdateFieldRequest request = NIMTeamUpdateFieldRequest();
    request.setName(name);
    return _updateTeam(teamId, request);
  }

  /// 更新群介绍
  static Future<bool> updateTeamIntroduce(String teamId, String introduction) {
    NIMTeamUpdateFieldRequest request = NIMTeamUpdateFieldRequest();
    request.setIntroduce(introduction);
    return _updateTeam(teamId, request);
  }

  /// 更新群邀请模式
  static Future<bool> updateInviteMode(
      String teamId, NIMTeamInviteModeEnum inviteModeEnum) async {
    NIMTeamUpdateFieldRequest request = NIMTeamUpdateFieldRequest();
    request.setInviteMode(inviteModeEnum);
    return _updateTeam(teamId, request);
  }

  /// 更新群资料修改模式
  static Future<bool> updateTeamInfoPrivilege(
      String teamId, NIMTeamUpdateModeEnum updateModeEnum) async {
    NIMTeamUpdateFieldRequest request = NIMTeamUpdateFieldRequest();
    request.setTeamUpdateMode(updateModeEnum);
    return _updateTeam(teamId, request);
  }

  /// 更新群扩展字段
  static Future<bool> updateTeamExtension(String tid, String extension) {
    NIMTeamUpdateFieldRequest request = NIMTeamUpdateFieldRequest();
    request.setExtension(extension);
    return _updateTeam(tid, request);
  }

  /// 更新入群模式
  static Future<bool> updateBeInviteMode(String teamId, bool isNeed) {
    NIMTeamUpdateFieldRequest request = NIMTeamUpdateFieldRequest();
    request.setBeInviteMode(isNeed
        ? NIMTeamBeInviteModeEnum.needAuth
        : NIMTeamBeInviteModeEnum.noAuth);
    return _updateTeam(teamId, request);
  }

  /// 退出群
  static Future<bool> quitTeam(String teamId) async {
    var teamResult = await NimCore.instance.teamService.queryTeam(teamId);
    // 讨论组的退出逻辑
    //1，判断是否为讨论组，非讨论组直接退出
    //2, 判断是否为创建者，非创建者直接退出
    //3, 判断是否成员数大于1，如果不大于1直接解散
    //4，请求成员列表，将群转给第一个成员，并退出
    if (getIt<TeamProvider>().isGroupTeam(teamResult.data)) {
      var team = teamResult.data;
      if (team?.creator == getIt<LoginService>().userInfo?.userId) {
        if ((team?.memberCount ?? 0) > 1) {
          var members =
              (await NimCore.instance.teamService.queryMemberList(teamId)).data;
          if (members?.isNotEmpty == true) {
            var newOwner = members?.firstWhere((element) =>
                element.account != getIt<LoginService>().userInfo?.userId);
            if (newOwner != null) {
              var resTrans = await NimCore.instance.teamService
                  .transferTeam(teamId, newOwner.account!, true);
              return resTrans.isSuccess;
            }
          }
        } else {
          var resDismiss =
              await NimCore.instance.teamService.dismissTeam(teamId);
          return resDismiss.isSuccess;
        }
      }
    }
    var res = await NimCore.instance.teamService.quitTeam(teamId);
    return res.isSuccess;
  }

  /// 解散群，只有创建者有此权限
  static Future<bool> dismissTeam(String teamId) async {
    var res = await NimCore.instance.teamService.dismissTeam(teamId);
    return res.isSuccess;
  }

  /// 会话是否置顶
  static Future<bool> isStickTop(String teamId) {
    return getIt<MessageProvider>().isStickSession(teamId, NIMSessionType.team);
  }

  /// 更新群中，用户的昵称
  static Future<bool> updateMemberNick(
      String teamId, String accId, String nickname) async {
    var res = await NimCore.instance.teamService
        .updateMemberNick(teamId, accId, nickname);
    return res.isSuccess;
  }

  /// 邀请用户入群
  static Future<NIMResult<List<String>>> inviteUser(
      String teamId, List<String> members) {
    return NimCore.instance.teamService.addMembersEx(
        teamId: teamId, accounts: members, msg: '', customInfo: '');
  }

  /// 会话置顶
  static Future<NIMStickTopSessionInfo?> addStickTop(
      String sessionId, String ext) {
    return getIt<MessageProvider>()
        .addStickTop(sessionId, NIMSessionType.team, ext);
  }

  /// 移除置顶
  static Future<bool> removeStickTop(String sessionId, String ext) {
    return getIt<MessageProvider>()
        .removeStick(sessionId, NIMSessionType.team, ext);
  }

  /// 添加管理员
  static Future<NIMResult<List<NIMTeamMember>>> addTeamManager(
      String tid, List<String> members) {
    return NimCore.instance.teamService.addManagers(tid, members);
  }

  /// 移除管理员
  static Future<NIMResult<List<NIMTeamMember>>> removeTeamManager(
      String tid, List<String> members) {
    return NimCore.instance.teamService.removeManagers(tid, members);
  }

  /// 移除群成员
  static Future<NIMResult<void>> removeTeamMembers(
      String tid, List<String> members) {
    return NimCore.instance.teamService.removeMembers(tid, members);
  }
}
